from sqlalchemy import Connection, text, select, func

from afeng_tools.sqlalchemy_tools.decorator.sqlalchemy_session_decorator import auto_conn, auto_commit_conn


@auto_conn()
def query_by_sql(sql: str, conn: Connection = None):
    """通过sql语句查找"""
    cursor = conn.execute(text(sql))
    return cursor.fetchall()


@auto_commit_conn()
def exec_by_sql(sql: str, conn: Connection = None):
    """通过sql运行"""
    cursor = conn.execute(text(sql))
    return cursor.lastrowid



"""
百度网盘用户工具
"""
import openapi_client
from openapi_client.api.userinfo_api import UserinfoApi
from afeng_tools.baidu_pan_tool.core.baidu_pan_decorator_tools import auto_userinfo_api
from afeng_tools.baidu_pan_tool.core.baidu_pan_models import UserInfo, PanVolumeInfo
from afeng_tools.log_tool.loguru_tools import log_error, get_logger

logger = get_logger()


@auto_userinfo_api
def get_user_info(access_token: str, api_instance: UserinfoApi = None) -> UserInfo:
    """
    获取用户信息：本接口用于获取用户的基本信息，包括账号、头像地址、会员类型等。
    :return:
    """
    try:
        api_response = api_instance.xpannasuinfo(access_token)
        if api_response['errno'] == 0:
            return UserInfo(
                baidu_name=api_response['baidu_name'],
                net_disk_name=api_response['netdisk_name'],
                avatar_url=api_response['avatar_url'],
                vip_type=api_response['vip_type'],
                uk=api_response['uk']
            )
        else:
            log_error(logger, '[BaiduPan]获取用户信息失败，api_response：' + api_response)
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling UserinfoApi->xpannasuinfo", e)


@auto_userinfo_api
def get_pan_info(access_token: str, check_free: int = 0, check_expire: int = 0,
                 api_instance: UserinfoApi = None) -> PanVolumeInfo:
    """
    获取网盘容量信息：本接口用于获取用户的网盘空间的使用情况，包括总空间大小，已用空间和剩余可用空间情况。
    :param access_token: Access Token
    :param check_free: 默认为0, 是否检查免费信息，0为不查，1为查
    :param check_expire: 是否检查过期信息，0为不查，1为查，默认为0
    :param api_instance: 自动注入的 UserinfoApi
    :return:
    """
    try:
        api_response = api_instance.apiquota(access_token, checkexpire=check_expire, checkfree=check_free)
        if api_response['errno'] == 0:
            return PanVolumeInfo(
                total=api_response['total'],
                expire=api_response['expire'],
                used=api_response['used'],
                free=api_response['free']
            )
        else:
            log_error(logger, '[BaiduPan]获取网盘空间失败，api_response：' + api_response)
    except openapi_client.ApiException as e:
        log_error(logger, "[BaiduPan]Exception when calling UserinfoApi->apiquota", e)



from afeng_tools.fastapi_tool.common.po_service.article_po_service_ import ArticlePoService
from afeng_tools.fastapi_tool.common.po_service.blacklist_po_service_ import BlacklistPoService
from afeng_tools.fastapi_tool.common.po_service.category_po_service_ import CategoryPoService
from afeng_tools.fastapi_tool.common.po_service.group_po_service_ import GroupPoService
from afeng_tools.fastapi_tool.common.po_service.link_po_service_ import LinkPoService
from afeng_tools.fastapi_tool.common.po_service.resource_po_service_ import ResourcePoService
from afeng_tools.fastapi_tool.common.po_service.sitemap_po_service_ import SitemapPoService
from afeng_tools.fastapi_tool.common.po_service.tag_po_service_ import TagPoService


class CommonPoService:
    """通用po服务"""

    def __init__(self, db_code: str):
        self.db_code = db_code

    def article_po_service(self) -> ArticlePoService:
        return ArticlePoService(self.db_code)

    def blacklist_po_service(self) -> BlacklistPoService:
        return BlacklistPoService(self.db_code)

    def category_po_service(self) -> CategoryPoService:
        return CategoryPoService(self.db_code)

    def group_po_service(self) -> GroupPoService:
        return GroupPoService(self.db_code)

    def link_po_service(self) -> LinkPoService:
        return LinkPoService(self.db_code)

    def resource_po_service(self) -> ResourcePoService:
        return ResourcePoService(self.db_code)

    def sitemap_po_service(self) -> SitemapPoService:
        return SitemapPoService(self.db_code)

    def tag_po_service(self) -> TagPoService:
        return TagPoService(self.db_code)



from typing import Optional

from fastapi import FastAPI
from fastapi.openapi.docs import get_swagger_ui_html, get_redoc_html
from starlette.requests import Request
from starlette.responses import JSONResponse, HTMLResponse

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum


def run_api_doc(app: FastAPI, openapi_url: Optional[str] = "/admin/openapi.json",
                docs_url: Optional[str] = "/admin/docs",
                redoc_url: Optional[str] = "/admin/redoc",
                swagger_ui_oauth2_redirect_url: Optional[str] = "/admin/oauth2-redirect"):
    app_config = {
        'title': settings_tools.get_config(SettingsKeyEnum.app_name),
        'openapi_url': openapi_url,
        'oauth2_redirect_url': swagger_ui_oauth2_redirect_url,
        'swagger_ui_init_oauth': None,
        'swagger_ui_parameters': None,
    }

    # openapi
    async def doc_openapi(req: Request) -> JSONResponse:
        return JSONResponse(app.openapi())
    app.add_route(openapi_url, doc_openapi, include_in_schema=False)

    # Swagger UI
    async def swagger_ui_html(req: Request) -> HTMLResponse:
        return get_swagger_ui_html(
            openapi_url=app_config['openapi_url'],
            title=str(app_config['title']) + " - Swagger页面",
            oauth2_redirect_url=app_config['oauth2_redirect_url'],
            init_oauth=app_config['swagger_ui_init_oauth'],
            swagger_ui_parameters=app_config['swagger_ui_parameters'],
            swagger_js_url="https://cdn.staticfile.org/swagger-ui/5.9.0/swagger-ui-bundle.min.js",
            swagger_css_url="https://cdn.staticfile.org/swagger-ui/5.9.0/swagger-ui.min.css",
            swagger_favicon_url="/static/image/favicon.ico",
        )
    app.add_route(docs_url, swagger_ui_html, include_in_schema=False)

    # ReDoc
    async def redoc_html(req: Request) -> HTMLResponse:
        return get_redoc_html(
            openapi_url=app_config['openapi_url'],
            title=str(app_config['title']) + " - ReDoc页面",
            redoc_js_url="https://cdn.jsdelivr.net/npm/redoc@next/bundles/redoc.standalone.js",
            redoc_favicon_url="/static/image/favicon.ico",
            with_google_fonts=False,
        )
    app.add_route(redoc_url, redoc_html, include_in_schema=False)



"""
邮箱工具
"""
import smtplib

smtp_server_dict = {
    '163': ('smtp.163.com', 25, 465),
    'aliyun': ('smtp.aliyun.com', 25, 465),
    'office365': ('smtp.office365.com', 587, 587)
}


def get_smtp_server_info(email_type: str) -> tuple[str, int, int]:
    """获取smtp服务器信息"""
    return smtp_server_dict.get(email_type)


def login_smtp_server(login_email: str, login_password: str,
                      smtp_host: str = None, smtp_port: int = 25, ssl_smtp_port: int = 465,
                      debug: bool = False, ssl: bool = False) -> smtplib.SMTP | smtplib.SMTP_SSL:
    """
    登录
    """
    if smtp_host is None:
        if login_email.endswith('@aliyun.com'):
            smtp_host, smtp_port, ssl_smtp_port = get_smtp_server_info('aliyun')
        if login_email.endswith('@163.com'):
            smtp_host, smtp_port, ssl_smtp_port = get_smtp_server_info('163')
    if ssl:
        # SMTP协议默认端口是25
        server = smtplib.SMTP_SSL(smtp_host, ssl_smtp_port)
    else:
        # SMTP协议默认端口是25
        server = smtplib.SMTP(smtp_host, smtp_port)
    # login()方法用来登录SMTP服务器
    server.login(login_email, login_password)
    if debug:
        # 打印出和SMTP服务器交互的所有信息。
        server.set_debuglevel(1)
    return server



import re
from copy import copy

from sqlalchemy import Column, Sequence
from sqlalchemy.orm import DeclarativeMeta

from afeng_tools.log_tool import loguru_tools
from afeng_tools.sqlalchemy_tool.core.sqlalchemy_db_tools import SqlalchemyDbTool


class ModelMetaClass(DeclarativeMeta):
    """模型类的元类"""
    logger = loguru_tools.get_logger()

    def __new__(self, name, bases, attrs, **kwargs):
        self._check_class_name_(name, bases, attrs)
        if SqlalchemyDbTool.is_postgresql:
            self._init_sequence_(name, bases, attrs)
        super_new = super().__new__
        parents = [b for b in bases if isinstance(b, ModelMetaClass)]
        if not parents:
            return super_new(self, name, bases, attrs)
        # 将类字段添加到columns中
        column_dict = dict()
        for p in parents:
            for k, v in p.__dict__.items():
                if isinstance(v, Column):
                    column_dict[k] = v
        for k, v in attrs.items():
            if isinstance(v, Column):
                column_dict[k] = v
        new_class = super_new(self, name, bases, attrs, **kwargs)
        setattr(new_class, '__column_dict__', column_dict)
        return new_class

    @classmethod
    def _init_sequence_(cls, class_name, class_bases, class_attrs):
        """初始化序列"""
        parents = [b for b in class_bases if isinstance(b, ModelMetaClass)]
        if not parents:
            return
        if SqlalchemyDbTool.is_postgresql:
            sequence_name = class_attrs["__tablename__"] + '_id_seq'
            if 'id' not in class_attrs:
                class_attrs['id'] = copy(getattr(parents[0], 'id'))
            try:
                id_column = class_attrs['id']
                id_column.default = Sequence(name=sequence_name, start=1, increment=1, cycle=False)
            except Exception as ex:
                loguru_tools.log_error(cls.logger, f'创建序列[{sequence_name}]失败', ex)

    @classmethod
    def _check_class_name_(cls, class_name, class_bases, class_attrs):
        """检查类名"""
        parents = [b for b in class_bases if isinstance(b, ModelMetaClass)]
        if not parents:
            return
        if not re.match('[A-Z]', class_name):
            raise TypeError('Model类名[%s]请修改为首字母大写' % class_name)
        if '__doc__' not in class_attrs or len(class_attrs['__doc__'].strip(' \n')) == 0:
            raise TypeError('Model类[%s]中必须有文档注释，并且文档注释不能为空' % class_name)



import logging
import os.path
import time
import traceback

from afeng_tools.file_tool import file_tools
from afeng_tools.template_tool import template_tools


def init_file_info_dict(vue_path: str, relative_file_list: list[str]) -> dict[str, float]:
    """初始化文件信息字典"""
    result_file_info_dict = {}
    for tmp_relative_file in relative_file_list:
        if tmp_relative_file.endswith('.vue'):
            result_file_info_dict[tmp_relative_file] = os.stat(os.path.join(vue_path, tmp_relative_file)).st_mtime
    return result_file_info_dict


def list_update_file(vue_path: str, old_file_info_dict: dict[str, float]):
    """列出更新文件"""
    relative_file_list, absolute_file_list = file_tools.list_files(vue_path, recursion=True,
                                                                   include_folder=False)
    result_file_list = []
    for tmp_relative_file in relative_file_list:
        if tmp_relative_file.endswith('.vue') and old_file_info_dict.get(tmp_relative_file):
            if os.stat(os.path.join(vue_path, tmp_relative_file)).st_mtime > old_file_info_dict[tmp_relative_file]:
                result_file_list.append(tmp_relative_file)
    return result_file_list


def get_vue_file_info_list(vue_path: str) -> dict[str, float]:
    """获取vue文件更新时间列表"""
    relative_file_list, _ = file_tools.list_files(vue_path, recursion=True, include_folder=False)
    return init_file_info_dict(vue_path, relative_file_list)


def render_vue_to_template(vue_file, template_file, template_root_path: str):
    """渲染 vue 模板为html模板"""
    html_code, js_code, css_code = template_tools.render(vue_file, template_root_path=template_root_path)
    template_content = css_code + html_code + js_code
    with open(template_file, 'w', encoding='utf-8') as tmp_file:
        tmp_file.write(template_content)
    return template_content


def render_vues_to_templates(vue_path: str, html_path: str, old_file_info_dict: dict[str, float],
                             template_root_path: str):
    """渲染 路径中的vue 模板为html模板"""
    vue_file_list = list_update_file(vue_path, old_file_info_dict)
    result_flag = False
    for tmp_vue_file in vue_file_list:
        tmp_html_file_name = tmp_vue_file.rsplit('.', maxsplit=1)[0] + '.html'
        tmp_html_file = os.path.join(os.path.join(html_path, tmp_html_file_name))
        os.makedirs(os.path.dirname(tmp_html_file), exist_ok=True)
        try:
            render_vue_to_template(os.path.join(vue_path, tmp_vue_file), tmp_html_file,
                                   template_root_path=template_root_path)
            print(f'成功渲染[{tmp_vue_file}]为[{tmp_html_file}]')
        except Exception as ex:
            logging.error(f'失败渲染[{tmp_vue_file}]为[{tmp_html_file}]:{ex}\n {traceback.format_exc()}')
        result_flag = True
    return result_flag


def refresh_render(vue_path: str, html_path: str, template_root_path: str):
    """刷新渲染"""
    relative_file_list, _ = file_tools.list_files(vue_path, recursion=True, include_folder=False)
    for tmp_relative_file in relative_file_list:
        if tmp_relative_file.endswith('.vue') and not os.path.exists(os.path.join(html_path, tmp_relative_file).removesuffix('.vue') + '.html'):
            tmp_vue_file = os.path.join(vue_path, tmp_relative_file)
            tmp_html_file = os.path.join(html_path, tmp_relative_file).removesuffix('.vue') + '.html'
            os.makedirs(os.path.dirname(tmp_html_file), exist_ok=True)
            try:
                render_vue_to_template(tmp_vue_file,
                                       tmp_html_file,
                                       template_root_path=template_root_path)
                print(f'成功渲染[{tmp_vue_file}]为[{tmp_html_file}]')
            except Exception as ex:
                logging.error(f'失败渲染[{tmp_vue_file}]为[{tmp_html_file}]:{ex}\n {traceback.format_exc()}')


def run_auto_render(vue_path: str, html_path: str, template_root_path: str):
    """运行自动渲染"""
    file_info_dict = get_vue_file_info_list(vue_path)
    print('启动自动渲染程序...')
    while True:
        render_flag = render_vues_to_templates(vue_path, html_path, file_info_dict,
                                               template_root_path=template_root_path)
        if render_flag:
            file_info_dict.update(get_vue_file_info_list(vue_path))
        else:
            new_file_info_dict = get_vue_file_info_list(vue_path)
            new_file_info_dict.update(file_info_dict)
            file_info_dict = new_file_info_dict
        refresh_render(vue_path, html_path, template_root_path)
        time.sleep(1)


if __name__ == '__main__':
    # pc端
    vue_template_path = r'C:\pythonwork\afeng-bookroom\resource\template\pc\component'
    html_template_path = r'C:\pythonwork\afeng-bookroom\resource\template\pc\tmp'
    # 手机端
    mobile_vue_template_path = r'C:\pythonwork\afeng-bookroom\resource\template\mobile\component'
    mobile_html_template_path = r'C:\pythonwork\afeng-bookroom\resource\template\mobile\tmp'
    template_roo = r'C:\pythonwork\afeng-bookroom\resource\template'
    # run_auto_render(vue_template_path, html_template_path, template_root_path=template_roo)
    run_auto_render(mobile_vue_template_path, mobile_html_template_path, template_root_path=template_roo)



"""
文章拆分工具
"""


def split_to_sentence(paragraph_text: str) -> list[str]:
    """
    段落拆分为句子（以"。"为分割句子）
    :param paragraph_text: 段落文本
    :return: 句子列表
    """
    sentence_list = []
    for tmp_sentence in paragraph_text.split('。'):
        if tmp_sentence.strip():
            sentence_list.append(tmp_sentence.strip())
    return sentence_list


def split_to_paragraph(article_text: str) -> list[str]:
    """
    把文章拆分为段落(以 换行符"\n"分割为段落)
    :param article_text: 文章文本
    :return: 段落列表
    """
    paragraph_list = []
    for tmp_paragraph in article_text.split('\n'):
        if tmp_paragraph.strip():
            paragraph_list.append(tmp_paragraph.strip())
    return paragraph_list


def replace_special_char(article_text: str) -> str:
    """
    替换特殊字符
    :param article_text:
    :return: 替换后的文本
    """
    text = article_text.replace('-', '')
    text = text.replace(' ', '')
    text = text.replace('’', '')
    text = text.replace('﨑', '崎')
    text = text.replace("[", ' ')
    text = text.replace("]", ' ')
    text = text.replace('　', ' ')
    text = text.replace("，]", "")
    text = text.replace("１", "1")
    text = text.replace("２", '2')
    text = text.replace("６", "6")
    text = text.replace("〔", "")
    text = text.replace("─", "")
    text = text.replace("┬", "")
    text = text.replace("┼", "")
    text = text.replace("┴", "")
    text = text.replace("〖", " ")
    text = text.replace("〗", " ")
    text = text.replace("礻殳", "祋")
    return text


def split_article(article_text: str) -> list[list[str]]:
    """
    拆分文章为句子
    :param article_text: 文章文本
    :return: list[list[句子]]
    """
    paragraph_list = split_to_paragraph(replace_special_char(article_text))
    result_list = []
    for tmp_paragraph in paragraph_list:
        result_list.append(split_to_sentence(tmp_paragraph))
    return result_list



"""
微信小程序工具
"""
from afeng_tools.encryption_tool import hashlib_tools
from afeng_tools.weixin_min_program.core.weixin_mp_decrypt import WXBizDataCrypt


def calc_signature(session_key: str, raw_data: str):
    """
    计算signature ： https://developers.weixin.qq.com/miniprogram/dev/framework/open-ability/signature.html
    signature = sha1( rawData + session_key )
    :return:
    """
    return hashlib_tools.calc_sha1(raw_data + session_key)


def check_signature(signature: str, session_key: str, raw_data: str):
    """数据签名校验"""
    signature_value = calc_signature(session_key, raw_data)
    return signature == signature_value


def decrypt_data(app_id: str, session_key: str, encrypted_data: str, iv_value: str):
    """解密数据"""
    return WXBizDataCrypt(app_id, session_key).decrypt(encrypted_data, iv_value)


if __name__ == '__main__':
    check_sign = '75e81ceda165f4ffa64f4068af58c64b8f54b88c'
    sign = calc_signature(session_key='HyVFkGl5F5OQWJZZaNzBBg==',
                          raw_data='{"nickName":"Band","gender":1,"language":"zh_CN","city":"Guangzhou","province":"Guangdong","country":"CN","avatarUrl":"http://wx.qlogo.cn/mmopen/vi_32/1vZvI39NWFQ9XM4LtQpFrQJ1xlgZxx3w7bQxKARol6503Iuswjjn6nIGBiaycAjAtpujxyzYsrztuuICqIM5ibXQ/0"}'
                          )
    if check_sign == sign:
        print('success')



import hashlib
from typing import Any

from fastapi import Query
from starlette.requests import Request

from afeng_tools.weixin_tool import weixin_settings
from afeng_tools.weixin_tool.core.model.item.wx_receive_event_models import WeixinEventItem
from afeng_tools.weixin_tool.core.model.item.wx_receive_msg_models import WeixinImageMsgItem, WeixinTextMsgItem, \
    WeixinVoiceMsgItem, \
    WeixinVideoMsgItem, WeixinLocationMsgItem, WeixinLinkMsgItem
from afeng_tools.weixin_tool.core.parse import parse_receive
from afeng_tools.weixin_tool.core.weixin_enums import WeixinConfigKeyEnum


async def convert_params(request: Request, signature: str = Query(title='微信加密签名'),
                         timestamp: str = Query(title='时间戳'),
                         nonce: str = Query(title='随机数'),
                         openid: str = Query(title='OpenID')) -> tuple[
    WeixinTextMsgItem | WeixinImageMsgItem | WeixinVoiceMsgItem | WeixinVideoMsgItem | WeixinLocationMsgItem | WeixinLinkMsgItem | WeixinEventItem, bytes,
    dict[str, Any]]:
    """格式化参数"""
    if signature == hashlib.sha1(
            ''.join(sorted([weixin_settings.get_config(WeixinConfigKeyEnum.weixin_token), timestamp, nonce])).encode(
                'utf-8')).hexdigest():
        # 证明请求来自微信服务器
        if request.headers['content-type'] == 'text/xml':
            body = await request.body()
            query_params = {
                'signature': signature,
                'timestamp': timestamp,
                'nonce': nonce,
                'openid': openid
            }
            return parse_receive.parse(openid, body), body, query_params



from typing import Callable, Optional, Union

from playwright.async_api import async_playwright, ProxySettings, Browser, BrowserContext, Page

from afeng_tools.decorator_tool.decorator_tools import run_async_func


def auto_input(input_type: Union[Browser, BrowserContext, Page] = Page, headless: bool = True,
               timeout: float = 1000 * 60 * 10, proxy: Optional[ProxySettings] = None):
    """注解：自动化注入 web_browser:Browser | web_context:BrowserContext | web_page:Page, 默认注入Page"""
    def auto_wrapper(func: Callable):
        async def func_wrapper(*args, **kwargs):
            if input_type == Browser and 'web_browser' in kwargs:
                return await run_async_func(func, *args, **kwargs)
            if input_type == BrowserContext and 'web_context' in kwargs:
                return await run_async_func(func, *args, **kwargs)
            if input_type == Page and 'web_page' in kwargs:
                return await run_async_func(func, *args, **kwargs)

            async with async_playwright() as pw:
                browser = await pw.chromium.launch(headless=headless, timeout=timeout, proxy=proxy)
                context = None
                page = None
                try:
                    if input_type == Browser:
                        kwargs['web_browser'] = browser
                    if input_type == BrowserContext:
                        context = await browser.new_context()
                        kwargs['web_context'] = context
                    if input_type == Page:
                        context = await browser.new_context()
                        page = await context.new_page()
                        kwargs['web_page'] = page
                    return await run_async_func(func, *args, **kwargs)
                finally:
                    if page:
                        await page.close()
                    if context:
                        await context.close()
                    await browser.close()
        return func_wrapper
    return auto_wrapper




from sqlalchemy import BigInteger, Column, Text, String, Enum, Boolean, DateTime, func, Float
from afeng_tools.fastapi_tool.common.enum import IconTypeEnum, ResourceFormatEnum, SitemapFreqEnum
from afeng_tools.sqlalchemy_tools.core import sqlalchemy_base_model


class CommonPo:
    """
    通用模型: 使用示例：

        GroupInfoPo, CategoryInfoPo, ResourceInfoPo, LinkInfoPo, TagInfoPo, BlacklistInfoPo, SitemapInfoPo, HelpInfoPo = CommonPo(db_code=app_info.db_code).create_models()
    """

    def __init__(self, db_code: str):
        self.db_code = db_code
        self.BaseModel = sqlalchemy_base_model.get_base_model(db_code=db_code)

    def create_group_po(self) -> type:
        class GroupInfoPo(self.BaseModel):
            """分组信息"""
            __tablename__ = "tb_group_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型',
                               unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)

        return GroupInfoPo

    def create_category_po(self) -> type:
        class CategoryInfoPo(self.BaseModel):
            """分类信息"""
            __tablename__ = "tb_category_info"
            group_code = Column(BigInteger, comment='分组编码', unique=False, nullable=True)
            code = Column(BigInteger, comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型',
                               unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
            parent_code = Column(BigInteger, comment='父编码', unique=False, nullable=True)

        return CategoryInfoPo

    def create_resource_po(self) -> type:
        class ResourceInfoPo(self.BaseModel):
            """资源信息"""
            __tablename__ = "tb_resource_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            resource_code = Column(BigInteger, comment='资源编码', index=True, unique=True, nullable=True)
            resource_name = Column(String(255), comment='资源名称', unique=False, nullable=True)
            resource_format = Column(Enum(ResourceFormatEnum), comment='资源格式（image:图片）', unique=False,
                                     nullable=True)
            local_path = Column(String(500), comment='本地路径', unique=False, nullable=True)
            baidu_fs_id = Column(BigInteger, comment='百度fs_id', unique=False, nullable=True)
            access_url = Column(String(500), comment='访问url', unique=False, nullable=True)
            expire_timestamp = Column(BigInteger, comment='过期时间戳', unique=False, nullable=True)
            download_flag = Column(Boolean, comment='下载标志', default=False)

        return ResourceInfoPo

    def create_link_po(self) -> type:
        class LinkInfoPo(self.BaseModel):
            """链接信息"""
            __tablename__ = "tb_link_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(100), comment='标题', unique=False, nullable=True)
            description = Column(Text, comment='描述', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型',
                               unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
            link_url = Column(String(500), comment='链接地址', unique=False, nullable=True)

        return LinkInfoPo

    def create_tag_po(self) -> type:
        class TagInfoPo(self.BaseModel):
            """标签信息"""
            __tablename__ = "tb_tag_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=True, nullable=True, index=True)
            title = Column(String(100), comment='标题', unique=False, nullable=True)
            description = Column(String(255), comment='描述', unique=False, nullable=True)

        return TagInfoPo

    def create_blacklist_po(self) -> type:
        class BlacklistInfoPo(self.BaseModel):
            """黑名单信息"""
            __tablename__ = "tb_blacklist_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            type_value = Column(String(255), comment='类型值', unique=False, nullable=True)

        return BlacklistInfoPo

    def create_sitemap_po(self) -> type:
        class SitemapInfoPo(self.BaseModel):
            """站点地图信息"""
            __tablename__ = "tb_sitemap_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            loc_url = Column(String(500), comment='url地址', unique=False, nullable=True)
            # 0.0 ~ 1.0
            priority = Column(Float, comment='优先权', unique=False, nullable=True)
            last_mod = Column(String(100), comment='最新一次更新时间(YYYY-MM-DD或，YYYY-MM-DDThh:mm:ss)', unique=False,
                              nullable=True)
            change_freq = Column(Enum(SitemapFreqEnum), comment='更新频率',
                                 default=SitemapFreqEnum.daily, unique=False, nullable=True)
            parent_id = Column(BigInteger, comment='主键', unique=False, nullable=True)

        return SitemapInfoPo

    def create_article_po(self) -> type:
        class ArticleInfoPo(self.BaseModel):
            """帮助信息"""
            __tablename__ = "tb_article_info"
            type_code = Column(String(100), comment='类型编码', default='default', unique=False, nullable=True)
            code = Column(String(100), comment='编码', unique=False, nullable=True, index=True)
            title = Column(String(255), comment='标题', unique=False, nullable=True)
            sub_title = Column(String(255), comment='副标题', unique=False, nullable=True)
            description = Column(String(255), comment='描述', unique=False, nullable=True)
            icon_type = Column(Enum(IconTypeEnum, values_callable=lambda x: [i.value.value for i in x]),
                               comment='图标类型',
                               unique=False, nullable=True)
            icon_value = Column(Text, comment='图标值', unique=False, nullable=True)
            source_info = Column(String(255), comment='来源信息', unique=False, nullable=True)
            author_info = Column(String(255), comment='作者信息', unique=False, nullable=True)
            publish_time = Column(DateTime, comment='发布时间', default=func.now())
            content = Column(Text, comment='内容', unique=False, nullable=True)

        return ArticleInfoPo

    def create_models(self, include_group: bool = True,
                      include_category: bool = True,
                      include_resource: bool = True,
                      include_link: bool = True,
                      include_tag: bool = True,
                      include_blacklist: bool = True,
                      include_sitemap: bool = True,
                      include_article: bool = True) -> tuple[type, ...]:
        """
            创建通用models, 示例： GroupInfoPo, CategoryInfoPo, ResourceInfoPo, LinkInfoPo, TagInfoPo, BlacklistInfoPo, SitemapInfoPo, HelpInfoPo = CommonPo(db_code=app_info.db_code).create_models()
            :param include_group:
            :param include_category:
            :param include_resource:
            :param include_link:
            :param include_tag:
            :param include_blacklist:
            :param include_sitemap:
            :param include_article:
            :return:
            """
        type_list = []
        if include_group:
            type_list.append(self.create_group_po())
        if include_category:
            type_list.append(self.create_category_po())
        if include_resource:
            type_list.append(self.create_resource_po())
        if include_link:
            type_list.append(self.create_link_po())
        if include_tag:
            type_list.append(self.create_tag_po())
        if include_blacklist:
            type_list.append(self.create_blacklist_po())
        if include_sitemap:
            type_list.append(self.create_sitemap_po())
        if include_article:
            type_list.append(self.create_article_po())

        return tuple(type_list)



"""
YAML工具: 安装 pip install pyyaml -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import yaml


def read_yaml(yaml_file: str, encoding: str = 'utf-8') -> dict:
    """读取yaml"""
    with open(yaml_file, 'r', encoding=encoding) as file:
        return yaml.load(file, Loader=yaml.FullLoader)



"""
日志配置
"""
import os.path

from afeng_tools.file_tool.tmp_file_tools import get_user_tmp_dir
from afeng_tools.log_tool.logger_enums import LoggerConfigKeyEnum

__LOGGER_CONFIG_CACHE__ = {
    LoggerConfigKeyEnum.info_file: os.path.join(get_user_tmp_dir(), '{time}-info.log'),
    LoggerConfigKeyEnum.info_rotation: '50 MB',
    LoggerConfigKeyEnum.error_file: os.path.join(get_user_tmp_dir(), '{time}-error.log'),
    LoggerConfigKeyEnum.error_rotation: '50 MB'
}


def set_config(config_key: LoggerConfigKeyEnum, config_value: str):
    """设置配置"""
    __LOGGER_CONFIG_CACHE__[config_key] = config_value


def get_config(config_key: LoggerConfigKeyEnum) -> str:
    """获取配置"""
    return __LOGGER_CONFIG_CACHE__.get(config_key)


from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class ResourcePoService(PoService):
    """
    使用示例：resource_po_service = ResourcePoService(app_info.db_code, ResourceInfoPo)
    """
    _table_name_ = "tb_resource_info"
    pass



"""
hashlib加密工具
"""
import hashlib
import hmac
import os


def calc_unique_id(data_list: list, split_chat: str = '') -> str:
    """
    计算唯一id
    :param data_list: 字符串列表
    :param split_chat: 分割符
    :return:
    """
    data_list = [str(tmp) for tmp in filter(lambda x: x is not None, data_list)]
    return calc_md5(split_chat.join(data_list))


def calc_byte_md5(data_byte: bytes) -> str:
    """计算md5"""
    return hashlib.md5(data_byte).hexdigest()


def calc_md5(data_str: str) -> str:
    """计算md5"""
    return hashlib.md5(data_str.encode('utf-8')).hexdigest()


def calc_file_md5(data_file: str) -> str:
    """计算文件的md5"""
    with open(data_file, 'rb') as f:
        return hashlib.md5(f.read()).hexdigest().lower()


def calc_file_slice_md5(data_file: str, slice_length: int = 256 * 1024) -> str:
    """文件校验段的MD5，32位小写，校验段对应文件前256KB"""
    if os.path.getsize(data_file) >= slice_length:
        with open(data_file, 'rb') as f:
            return hashlib.md5(f.read(slice_length)).hexdigest().lower()
    else:
        return calc_file_md5(data_file=data_file)


def calc_hmac_sha512(secret_key: str, data: str):
    """
    计算sha512值
    :param secret_key: 加密的键
    :param data: 要加密的数据
    :return: sha512值
    """
    hash_func = hashlib.sha512
    return hmac.new(key=secret_key.encode('utf-8'),
                    msg=data.encode('utf-8'),
                    digestmod=hash_func).hexdigest().lower()


def calc_sha1(data_str: str):
    """
    计算sha1值
    :param data_str: 要加密的数据
    :return: sha512值
    """
    return hashlib.sha1(data_str.encode('utf-8')).hexdigest()


def calc_sha224(data_str: str):
    """
    计算sha1值
    :param data_str: 要加密的数据
    :return: sha512值
    """
    return hashlib.sha224(data_str.encode('utf-8')).hexdigest()


def calc_sha512(data_str: str):
    """
    计算sha512值
    :param data_str: 要加密的数据
    :return: sha512值
    """
    return hashlib.sha512(data_str.encode('utf-8')).hexdigest()



"""
分词工具
"""
from afeng_tools.text_tool.word_break_tool import jieba_tools


def word_cut(text_content: str) -> list[str]:
    return jieba_tools.word_cut(text_content)


def word_cut_for_search(text_content: str) -> list[str]:
    return jieba_tools.word_cut_for_search(text_content)



from enum import Enum
from typing import Optional, Callable

from afeng_tools.datetime_tool.datetime_tools import one_minute_seconds, one_hour_seconds, one_day_seconds, \
    one_week_seconds
from afeng_tools.pydantic_tool.model.common_models import EnumItem


class IconTypeEnum(Enum):
    """图标类型枚举"""
    font_icon = EnumItem(title='文字图标')
    base64_icon = EnumItem(title='Base64图标')
    svg_icon = EnumItem(title='svg图标')
    image_icon = EnumItem(title='图片图标')
    resource_icon = EnumItem(title='资源图标')


class ReloadFreqItem(EnumItem):
    """更新频率项"""
    # 秒值
    seconds_value: int
    # 判断函数：返回True则进行Reload， 如：def func (timestamp) -> bool
    judge_func: Optional[Callable[[float], bool]] = None


class ReloadFreqEnum(Enum):
    """更新频率枚举"""
    always = ReloadFreqItem(title='一直更新', seconds_value=0, judge_func=lambda x: True)
    half_hourly = ReloadFreqItem(title='半小时更新一次', seconds_value=30 * one_minute_seconds)
    hourly = ReloadFreqItem(title='每小时更新一次', seconds_value=one_hour_seconds)
    eight_hourly = ReloadFreqItem(title='8小时更新一次', seconds_value=8 * one_hour_seconds)
    daily = ReloadFreqItem(title='每天更新一次', seconds_value=one_day_seconds)
    weekly = ReloadFreqItem(title='每周更新一次', seconds_value=one_week_seconds)
    monthly = ReloadFreqItem(title='每月更新一次', seconds_value=30 * one_day_seconds)
    three_monthly = ReloadFreqItem(title='三个月更新一次', seconds_value=90 * one_day_seconds)
    half_yearly = ReloadFreqItem(title='半年更新一次', seconds_value=180 * one_day_seconds)
    yearly = ReloadFreqItem(title='每年更新一次', seconds_value=360 * one_day_seconds)
    never = ReloadFreqItem(title='从不更新', seconds_value=100 * 360 * one_day_seconds, judge_func=lambda x: False)


class SitemapFreqEnum(Enum):
    """站点更新频率"""
    always = 'always'
    hourly = 'hourly'
    daily = 'daily'
    weekly = 'weekly'
    monthly = 'monthly'
    yearly = 'yearly'
    never = 'never'


class ResourceFormatEnum(Enum):
    """资源格式枚举"""
    image = 'image'
    file = 'file'


class ResourceUrlEnum(Enum):
    """资源格式枚举"""
    # 资源根路径
    base_url = '/resource/public'
    # 资源下载路径
    download_url = '/resource/download'



import logging
import re
import traceback
from copy import copy

from sqlalchemy import Column, Sequence
from sqlalchemy.orm import DeclarativeMeta


def is_postgresql_db(bases):
    """判断是否是postgresql数据库"""
    result_flag = False
    parents = [b for b in bases if isinstance(b, ModelMetaClass)]
    if parents:
        for tmp_parent in parents:
            super_parents = [tmp for tmp in tmp_parent.__bases__ if tmp.__name__ == 'Base']
            for tmp_super_parent in super_parents:
                if tmp_super_parent.__dict__.get('is_postgresql'):
                    return True
    return result_flag


class ModelMetaClass(DeclarativeMeta):
    """模型类的元类"""

    def __new__(self, name, bases, attrs, **kwargs):
        self._check_class_name_(name, bases, attrs)
        if is_postgresql_db(bases):
            self._init_sequence_(name, bases, attrs)
        super_new = super().__new__
        parents = [b for b in bases if isinstance(b, ModelMetaClass)]
        if not parents:
            return super_new(self, name, bases, attrs)
        # 将类字段添加到columns中
        column_dict = dict()
        for p in parents:
            for k, v in p.__dict__.items():
                if isinstance(v, Column):
                    column_dict[k] = v
        for k, v in attrs.items():
            if isinstance(v, Column):
                column_dict[k] = v
        new_class = super_new(self, name, bases, attrs, **kwargs)
        setattr(new_class, '__column_dict__', column_dict)
        return new_class

    @classmethod
    def _init_sequence_(cls, class_name, class_bases, class_attrs):
        """初始化序列"""
        parents = [b for b in class_bases if isinstance(b, ModelMetaClass)]
        if not parents or '__tablename__' not in class_attrs or '__abstract__' in class_attrs:
            return
        sequence_name = class_attrs["__tablename__"] + '_id_seq'
        if 'id' not in class_attrs:
            class_attrs['id'] = copy(getattr(parents[0], 'id'))
        try:
            id_column = class_attrs['id']
            id_column.default = Sequence(name=sequence_name, start=1, increment=1, cycle=False)
        except Exception as ex:
            logging.error(f'创建序列[{sequence_name}]失败\n {traceback.format_exc()}')

    @classmethod
    def _check_class_name_(cls, class_name, class_bases, class_attrs):
        """检查类名"""
        parents = [b for b in class_bases if isinstance(b, ModelMetaClass)]
        if not parents:
            return
        if not re.match('[A-Z]', class_name):
            raise TypeError('Model类名[%s]请修改为首字母大写' % class_name)
        if '__doc__' not in class_attrs or len(class_attrs['__doc__'].strip(' \n')) == 0:
            raise TypeError('Model类[%s]中必须有文档注释，并且文档注释不能为空' % class_name)



"""
- pip install Jinja2 -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import os

from jinja2 import Template, FileSystemLoader, Environment

from afeng_tools.decorator_tool import decorator_tools
from afeng_tools.jinja2_tool import jinja2_filters


def _auto_append_filter(env: Environment):
    """自动添加过滤器"""
    func_members = decorator_tools.get_func_by_decorator(jinja2_filters, 'jinja2_filter')
    if func_members:
        for tmp_member in func_members.values():
            env.filters[getattr(tmp_member, '__filter_name__')] = tmp_member
    return env


def create_template(content_list: list[str]) -> Template:
    """创建模板"""
    return Template('\n'.join(content_list))


def create_template_env(directory_list: list[str], auto_escape: bool = True) -> Environment:
    """获取模板环境"""
    result_env = Environment(loader=FileSystemLoader(directory_list),
                             autoescape=auto_escape)
    result_env = _auto_append_filter(result_env)
    return result_env


def format_template(template_file: str, context: dict = None, env: Environment = None,
                    template_path: str = None) -> str:
    """格式化模板"""
    if env is None:
        if template_path is None:
            template_path = os.path.dirname(template_file)
            template_file = os.path.split(template_file)[1]
        env = create_template_env([template_path])
    template = env.get_template(template_file)
    return template.render(**context)


def render_template(template: Template, context: dict = None):
    """格式Template"""
    return template.render(**context)


if __name__ == '__main__':
    print()



"""
pydantic 工具
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
- pip install Flask
- pip install flask_socketio
- pip install flask_babel
- pip install flask_login
- pip install flask_mail
- pip install flask_paranoid
- pip install flask_security
- pip install email_validator
- pip install flask_sqlalchemy
- pip install simplejson
- pip install flask_migrate

"""



"""
playwright工具：pip install pytest-playwright -i https://pypi.tuna.tsinghua.edu.cn/simple/ -U
"""
import time
from typing import Callable

from playwright.sync_api import Page, sync_playwright

from afeng_tools.playwright_tool import http_header_tools, http_cookie_tools
from afeng_tools.playwright_tool.decorator.playwright_decorators import auto_input


@auto_input(headless=True)
def refresh_cookie_and_header_file(refresh_url: str, cookie_file: str, header_file: str, web_page: Page = None,
                                   sleep_seconds: int = 1) -> None:
    """
    刷新cookie和header文件
    :param refresh_url: 属性数据的请求的url
    :param cookie_file: cookie文件路径
    :param header_file: header文件路径
    :param web_page: 自动注入的Page
    :param sleep_seconds: 为避免高频请求睡眠的秒数
    :return: None
    """
    http_header_tools.save_headers(web_page, header_file)
    print('*' * 10, refresh_url)
    web_page.goto(refresh_url)
    # 为避免高频请求导致网站封ip，需要睡眠一会，小于等于0则不会睡眠
    if sleep_seconds > 0:
        time.sleep(sleep_seconds)
    http_cookie_tools.save_cookies(web_page, cookie_file)


def run_page_work(headless: bool, work_func: Callable):
    """
    执行页面工作
    :param headless: 是否有请求头
    :param work_func: 执行工作的函数，必须要有参数:web_page:Page
    :return:
    """
    with sync_playwright() as pw:
        browser_args = ['--start-maximized']
        browser = pw.chromium.launch(headless=headless, timeout=1000 * 60 * 10, args=browser_args)
        page = browser.new_page(no_viewport=True)
        try:
            if isinstance(work_func, Callable):
                return work_func(web_page=page)
            else:
                return work_func.__func__(web_page=page)
        finally:
            if page:
                page.close()
            browser.close()





from enum import Enum
from typing import TypeVar, Type, Callable
from fastapi.exceptions import RequestValidationError
from starlette.requests import Request

from afeng_tools.pydantic_tool.model.common_models import EnumItem


def convert_enum_type(param_name: str, enum_class: Type[Enum]) -> Callable:
    """
    转换枚举类型的装饰器， 使用示例：search_type: SearchTypeEnum = Depends(convert_enum_type('search_type', SearchTypeEnum)
    :param param_name: 参数名称
    :param enum_class: 枚举类型
    :return: 枚举
    """
    EnumType = TypeVar(enum_class.__name__, bound=enum_class)

    def convert_enum_type_wrapper(request: Request) -> EnumType:
        enum_type_str = request.query_params.get(param_name)
        if not enum_type_str:
            enum_type_str = request.path_params.get(param_name)
            if not enum_type_str:
                enum_type_str = request.form().get(param_name)
        type_enum_list = [tmp for tmp in enum_class if tmp.name == enum_type_str or tmp.value == enum_type_str]
        if type_enum_list:
            return type_enum_list[0]
        else:
            type_enum_list = [tmp for tmp in enum_class if
                              isinstance(tmp.value, EnumItem) and tmp.value.value == enum_type_str]
            if type_enum_list:
                return type_enum_list[0]
        raise RequestValidationError('请求参数[search_type]值有误！')

    return convert_enum_type_wrapper



from typing import Callable

from afeng_tools.weixin_tool.core.weixin_enums import WeixinConfigKeyEnum, WeixinConfigItem

# 微信配置
__WEIXIN_CONFIG_CACHE__ = dict()


def init_config(weixin_config_item: WeixinConfigItem):
    init_weixin_config(weixin_config_item)


def init_weixin_config(weixin_config_item: WeixinConfigItem):
    """初始化微信公众号配置"""
    _set_config(WeixinConfigKeyEnum.weixin_app_id, weixin_config_item.weixin_app_id)
    _set_config(WeixinConfigKeyEnum.weixin_app_secret, weixin_config_item.weixin_app_secret)
    _set_config(WeixinConfigKeyEnum.weixin_token, weixin_config_item.weixin_token)
    _set_config(WeixinConfigKeyEnum.weixin_encoding_aes_key, weixin_config_item.weixin_encoding_aes_key)
    _set_config(WeixinConfigKeyEnum.weixin_msg_callback, weixin_config_item.weixin_msg_callback)
    _set_config(WeixinConfigKeyEnum.weixin_token_file, weixin_config_item.weixin_token_file)


def _set_config(config_key: WeixinConfigKeyEnum, config_value: str | Callable):
    """设置配置"""
    __WEIXIN_CONFIG_CACHE__[config_key] = config_value


def get_config(config_key: WeixinConfigKeyEnum) -> str | Callable:
    """获取配置"""
    return __WEIXIN_CONFIG_CACHE__.get(config_key)



from typing import Optional

from pydantic import BaseModel


class WeixinButtonItem(BaseModel):
    """根菜单项（用于具有子菜单的菜单项）"""
    name: str
    sub_button: list


class WeixinClickButtonItem(BaseModel):
    """click：点击推事件用户点击click类型按钮后，微信服务器会通过消息接口推送消息类型为event的结构给开发者（参考消息接口指南），并且带上按钮中开发者填写的key值，开发者可以通过自定义的key值与用户进行交互；
    """
    type: str = 'click'
    name: str
    key: str


class WeixinViewButtonItem(BaseModel):
    """view:跳转URL用户点击view类型按钮后，微信客户端将会打开开发者在按钮中填写的网页URL，可与网页授权获取用户基本信息接口结合，获得用户基本信息。
    """
    type: str = 'view'
    name: str
    url: str


class WeixinMiniProgramButtonItem(BaseModel):
    """miniprogram:用户点击miniprogram类型按钮后，微信客户端将会打开开发者在按钮中填写微信小程序页面。
    """
    type: str = 'miniprogram'
    name: str
    url: str = 'http://mp.weixin.qq.com'
    # 小程序的appid（仅认证公众号可配置）
    appid: str
    # 小程序的页面路径
    pagepath: str


class WeixinScanPushButtonItem(BaseModel):
    """scancode_push: 扫码推事件用户点击按钮后，微信客户端将调起扫一扫工具，完成扫码操作后显示扫描结果（如果是URL，将进入URL），且会将扫码的结果传给开发者，开发者可以下发消息。
    """
    type: str = 'scancode_push'
    name: str
    key: str
    sub_button: Optional[list] = []


class WeixinScanWaitButtonItem(BaseModel):
    """scancode_waitmsg：扫码推事件且弹出“消息接收中”提示框用户点击按钮后，微信客户端将调起扫一扫工具，完成扫码操作后，将扫码的结果传给开发者，同时收起扫一扫工具，然后弹出“消息接收中”提示框，随后可能会收到开发者下发的消息。
    """
    type: str = 'scancode_waitmsg'
    name: str
    key: str
    sub_button: Optional[list] = []


class WeixinPhotoButtonItem(BaseModel):
    """pic_sysphoto：弹出系统拍照发图用户点击按钮后，微信客户端将调起系统相机，完成拍照操作后，会将拍摄的相片发送给开发者，并推送事件给开发者，同时收起系统相机，随后可能会收到开发者下发的消息。
    """
    type: str = 'pic_sysphoto'
    name: str
    key: str
    sub_button: Optional[list] = []


class WeixinPicButtonItem(BaseModel):
    """pic_photo_or_album：弹出拍照或者相册发图用户点击按钮后，微信客户端将弹出选择器供用户选择“拍照”或者“从手机相册选择”。用户选择后即走其他两种流程。
    """
    type: str = 'pic_photo_or_album'
    name: str
    key: str
    sub_button: Optional[list] = []


class WeixinPicWxButtonItem(BaseModel):
    """pic_weixin：弹出微信相册发图器用户点击按钮后，微信客户端将调起微信相册，完成选择操作后，将选择的相片发送给开发者的服务器，并推送事件给开发者，同时收起相册，随后可能会收到开发者下发的消息。
    """
    type: str = 'pic_weixin'
    name: str
    key: str
    sub_button: Optional[list] = []


class WeixinLocationButtonItem(BaseModel):
    """location_select：弹出地理位置选择器用户点击按钮后，微信客户端将调起地理位置选择工具，完成选择操作后，将选择的地理位置发送给开发者的服务器，同时收起位置选择工具，随后可能会收到开发者下发的消息。
    """
    type: str = 'location_select'
    name: str
    key: str


class WeixinMediaButtonItem(BaseModel):
    """media_id：下发消息（除文本消息）用户点击media_id类型按钮后，微信服务器会将开发者填写的永久素材id对应的素材下发给用户，永久素材类型可以是图片、音频、视频 、图文消息。请注意：永久素材id必须是在“素材管理/新增永久素材”接口上传后获得的合法id。
    """
    type: str = 'media_id'
    name: str
    media_id: str


class WeixinArticleButtonItem(BaseModel):
    """article_id：用户点击 article_id 类型按钮后，微信客户端将会以卡片形式，下发开发者在按钮中填写的图文消息
    """
    type: str = 'article_id'
    name: str
    article_id: str


class WeixinArticleViewButtonItem(BaseModel):
    """article_view_limited：类跳转图文消息URL用户点击article_view_limited类型按钮后，微信客户端将打开开发者在按钮中article_id对应的图文消息URL。
    """
    type: str = 'article_view_limited'
    name: str
    # 发布后获得的合法 article_id
    article_id: str


if __name__ == '__main__':
    print(WeixinClickButtonItem(name='test', key='测试').model_copy())



"""
爬虫item
"""
from typing import Optional

from pydantic import BaseModel


class WikiInfoRuleItem(BaseModel):
    """Wiki信息规则项"""
    # 文本规则
    title_rule: Optional[str] = None
    # 链接规则
    url_rule: Optional[str] = None
    # url请求的页面内容规则
    page_content_rule: Optional[str] = None
    # 子项列表规则
    child_item_list_rule: Optional[str] = None
    # 子项信息规则项
    child_item_info_rule: Optional['WikiInfoRuleItem'] = None
    # 是否递归
    is_recursion: bool = False


class WikiItemResult(BaseModel):
    """wiki爬虫单项结果"""
    # 标题列表，是分级别的
    title_list: Optional[list[str]] = []
    # 标题路径列表，对应上面的标题列表
    url_list: Optional[list[str]] = []
    # 标题路径列表，对应上面的标题列表
    source_url_list: Optional[list[str]] = []
    # 页面内容获取规则
    content_rule: Optional[str] = None
    # 保存的html文件名
    html_file_name: Optional[str] = None
    # html路径锚点
    html_url_anchor: Optional[str] = None





import requests


def get_json(url: str, params: dict = None, headers: dict = None, cookies: dict = None, encoding: str = 'utf-8'):
    response = requests.get(url, params=params, headers=headers, cookies=cookies)
    if response.status_code == 200:
        response.encoding = encoding
        return response.text


def post_json(url: str, headers: dict = None, params: dict = None, files: dict = None,
              cookies: dict = None, data: dict = None, json: dict = None, encoding: str = 'utf-8'):
    response = requests.post(url, headers=headers, params=params, data=data, json=json, files=files, cookies=cookies)
    if response.status_code == 200:
        response.encoding = encoding
        return response.json()


if __name__ == '__main__':
    result = post_json('http://www.zmtt.net/checkUpdate', json={"version": "2.0"})
    print(result)



"""
pandas工具： pip install pandas
"""
from pandas import DataFrame


def get_size(df: DataFrame):
    row_count, col_count = df.shape
    return row_count, col_count


def get_columns(df: DataFrame):
    return df.columns


def get_index(df: DataFrame):
    return df.index


def get_values(df: DataFrame):
    return df.values


def get_cols_by_column(df: DataFrame, column_list: list):
    return df[column_list]


def get_cols_by_column2(df: DataFrame, column_list: list):
    return df.loc[:, column_list]


def get_cols_by_index(df: DataFrame, col_index_list: list):
    """获取第col_index_list列的值"""
    return df.iloc[:col_index_list]


def get_rows_by_index_name(df: DataFrame, index_name_list: list):
    return df.loc[index_name_list]


def get_rows_by_index(df: DataFrame, index_list: list):
    return df.iloc[index_list]



from afeng_tools.qrcode_tool.qrcode_tools import make_qrcode_image


def print_qrcode_info(data_str: str):
    """测试打印二维码"""
    img = make_qrcode_image(data_str, 'output/test,png')
    img.show()
    print('box_size:', img.box_size, '\n',
          'border:', img.border, '\n',
          '前景色：', img.fill_color, '\n'
                                     '图片像素大小：', img.pixel_size, '\n',
          '图片类型:', img.kind, '\n',  # 效果等同于img.check_kind(kind=None)
          'version:', int(1 + (img.width - 21) / 4)  # img.width的值就是每行或每列的box的数量
          )


if __name__ == '__main__':
    print_qrcode_info('测试')


"""
- pip install pydantic -i https://pypi.tuna.tsinghua.edu.cn/simple/
"""
from enum import Enum

from pydantic import BaseModel, Field


class MobileAppInfo(BaseModel):
    """手机应用信息"""
    app_name: str = Field(title="应用名")
    app_package: str = Field(title="应用package")
    app_activity: str = Field(title="应用activity")


class MobileAppInfoEnum(Enum):
    bai_jia_hao = MobileAppInfo(app_name='百家号', app_package='com.baidu.baijia',
                                app_activity='.splash.SplashActivity')
    bi_jian = MobileAppInfo(app_name='必剪', app_package='com.bilibili.studio',
                            app_activity='com.bcut.homepage.widget.MainActivity')
    bili_bili = MobileAppInfo(app_name='哔哩哔哩', app_package='tv.danmaku.bili', app_activity='.MainActivityV2')
    dou_yin = MobileAppInfo(app_name='抖音', app_package='com.ss.android.ugc.aweme',
                            app_activity='.splash.SplashActivity')
    kuai_shou = MobileAppInfo(app_name='快手', app_package='com.smile.gifmaker',
                              app_activity='com.yxcorp.gifshow.HomeActivity')
    qi_e_hao = MobileAppInfo(app_name='企鹅号', app_package='com.tencent.omapp',
                             app_activity='.ui.activity.SplashActivity')
    qq = MobileAppInfo(app_name='QQ', app_package='com.tencent.mobileqq',
                       app_activity='com.tencent.mobileqq.activity.SplashActivity')
    tao_bao = MobileAppInfo(app_name='淘宝', app_package='com.taobao.taobao',
                            app_activity='com.taobao.tao.welcome.Welcome')
    wei_xin = MobileAppInfo(app_name='微信', app_package='com.tencent.mm', app_activity='.ui.LauncherUI')
    xiao_hong_shu = MobileAppInfo(app_name='小红书', app_package='com.xingin.xhs',
                                  app_activity='.index.v2.IndexActivityV2')
    xi_gua_shi_pin = MobileAppInfo(app_name='西瓜视频', app_package='com.ss.android.article.video',
                                   app_activity='.activity.SplashActivity')
    zhi_fu_bao = MobileAppInfo(app_name='支付宝', app_package='com.eg.android.AlipayGphone',
                               app_activity='.AlipayLogin')
    jing_dong_reader = MobileAppInfo(app_name='京东读书', app_package='com.jd.app.reader',
                                     app_activity='com.jingdong.app.reader.logo.JdLogoActivity')



"""
http枚举
"""
from enum import Enum


class HttpMethodEnum(Enum):
    post = 'POST'
    get = 'GET'
    put = 'PUT'
    delete = 'DELETE'
    head = 'HEAD'
    options = 'OPTIONS'
    patch = 'PATCH'


class HttpContentTypeEnum(Enum):
    form_urlencoded = 'application/x-www-form-urlencoded;charset=utf-8'
    form_data = 'application/x-www-form-urlencoded, multipart/form-data'
    json = 'application/json;charset=utf-8'
    xml = 'application/xml'
    text = 'text/plain'

